/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.monitors;

//---IMPORTS---
import java.util.HashMap;
import java.text.NumberFormat;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This Monitor watches the Swap for all machines
 *
 * @author  $Author: tdb $
 * @version $Id: Swap__Monitor.java,v 1.16 2004/08/01 10:40:45 tdb Exp $
 */
public class Swap__Monitor extends MonitorSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.16 $";
    
    /**
     * A description of this monitor
     */
    public final String DESC = "Monitors Swap.";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---
    
    /**
     * Analyse a packet of data, and generate an alert if
     * necessary.
     *
     * @param packet the XMLPacket to analyse
     */
    public void analysePacket(XMLPacket packet) {
        String source = packet.getParam("packet.attributes.machine_name");
        if(!checkBooleanConfig("Host." + source, "Monitor." + _name + ".enable")) {
            return;
        }
        if (!_hosts.containsKey(source)) {
            _hosts.put(source, new Register(source, _name));
        }

        Register reg = (Register) _hosts.get(source);

        // get the packet data
        double swapTotal, swapFree;
        try {
            String total = packet.getParam("packet.swap.total");
            String free = packet.getParam("packet.swap.free");
            if(total==null || free==null) {
                throw new NumberFormatException("Swap data invalid");
            }
            swapTotal = Double.parseDouble(total);
            swapFree = Double.parseDouble(free);
        } catch (NumberFormatException e) {
            _logger.write(this.toString(), Logger.WARNING, "Received packet from "+source+" with bad swap information: "+e);
            // don't try to continue and process
            return;
        }

        boolean useValue = false;
        try {
           String option = _cp.getProperty("Host." + source, "Monitor." + _name + ".thresholdMeasure");
           if (option.equals("VALUE")) {
               useValue = true;
           }
        } catch (PropertyNotFoundException e) {
            // we default to percentage
        }

        // this bit determines if the swap check is a % check
        // or a byte check
        String type;
        double curValue;
        int newThreshold;
        if(useValue) {
            // bytes of swap available
            curValue = swapFree;
            // negate check
            newThreshold = checkAttributeThreshold(curValue, reg, true);
            type = "bytes";
        } else {
            // % memory in use
            curValue = (1 - ((double)swapFree / (double)swapTotal)) * 100;
            // normal check
            newThreshold = checkAttributeThreshold(curValue, reg, false);
            type = "%";
        }

        // format the value to a String
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        nf.setMinimumFractionDigits(2);
        String strSwapInUse = nf.format(curValue);

        String attributeName = "Swap in use " + type;

        processAlert(newThreshold, attributeName, reg, source, strSwapInUse);
            
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * return the String representation of what the monitor does
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---
    
    /**
     * Checks a piece of current data, and returns the
     * threshold it breaches, if any.
     *
     * The option to negate the check can be used in
     * situations where being *below* the threshold
     * is an 'alertable' situation. In this specific
     * case, we'd do this with kb disk checks.
     *
     * @param value the current value
     * @param reg the Register for the host
     * @param negateCheck whether to negate the check
     * @return the threshold level breached, if any
     */
    private int checkAttributeThreshold(double value, Register reg, boolean negateCheck) {
        for(int thresholdLevel = Alert.thresholdLevels.length - 1; thresholdLevel >= 0; thresholdLevel--) {
            if (reg.getThreshold(thresholdLevel) != -1.0) {
                if(!negateCheck) {
                    // normal check - has the value gone *over* the threshold
                    if(((double) reg.getThreshold(thresholdLevel)) < value) {
                        return thresholdLevel;
                    }
                }
                else {
                    // negated check - has the value gone *under* the threshold
                    if(((double) reg.getThreshold(thresholdLevel)) > value) {
                        return thresholdLevel;
                    }
                }
            }
        }
        return Alert.thresholdNORMAL;
    }

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns a reference to a specific Queue for this
     * monitor. This Queue returns only the data packets
     * (based on type) that we want too look at.
     *
     * @return a reference to a Queue
     */
    protected Queue getQueue() {
        return MonitorManager.getInstance().getDataQueue();
    }


//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = "Swap";

    /**
     * A reference to the configuration proxy in use
     */
    private ConfigurationProxy _cp = ConfigurationProxy.getInstance();

    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring.
     */
    private HashMap _hosts = new HashMap();

//---STATIC ATTRIBUTES---

}
