/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filter;

//---IMPORTS---
import java.io.*;
import java.util.*;

import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;

/**
 * This class setups up and manages Plugins for performing service
 * checks.  When asked to run some service checks it runs the checks
 * on the service check pipeline for that host.  If the pipleline does
 * not exist it creates a new one.
 *
 * This is a singleton class.
 *
 * ###
 * This does not fully manage the pipelines, it should check
 * if a host has not heartbeated for a while and thus remove 
 * the pipeline.  At the moment it does not.
 * A suggestions for doing this is to use the getCreated() 
 * method on the PluginServiceCheckPipeline.  This tells you when
 * according to the system time the pipeline was created.  You could
 * then have a thread that periodically checks to see if a host
 * has not run its pipeline in say two heartbeats, and thus
 * this indicates that it is no longer running.  The pipeline
 * could then be removed from the hashmap and thus be eligable
 * for gc.
 * ###
 *
 * @author  $Author: tdb $
 * @version $Id: PluginServiceCheckManager.java,v 1.10 2004/08/01 10:40:59 tdb Exp $
 */
class PluginServiceCheckManager {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.10 $";
    
    /**
     * file name suffix for plugin service check classes:
     */
    private final String _suffix = "__ServiceCheck";
    
//---STATIC METHODS---

    /**
     * Return a reference to the singleton class.
     * Construct it if it does not already exist, otherwise just return the reference.
     */
    public static PluginServiceCheckManager getInstance() {
        if (_instance == null){
            _instance = new PluginServiceCheckManager();
        }
        return _instance;
    }

//---CONSTRUCTORS---

    /**
     * Private Constructor - this part creates the filter pipeline
     * This is a singleton class, btw.
     */
    private PluginServiceCheckManager(){
        _logger.write(toString(), Logger.SYSINIT, "created");
    }

//---PUBLIC METHODS---

    /**
     * run all registered service checks on a given host,
     * returning XML data with information about the check.
     *
     * @param hostname the host to run service checks for
     * @return the xml output from the service checks
     */
    public String runServiceChecks(String hostname){
        if (!_hostPipelines.containsKey(hostname)) {
               _hostPipelines.put(hostname, new PluginServiceCheckPipeline(hostname, this));
        }
        return "<services>" + ((PluginServiceCheckPipeline) _hostPipelines.get(hostname)).runPipeline() + "</services>";
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    /**
     * Returns a reference to the specified service check.
     * This method takes the name of the class that contains
     * a service check implementation.  It then checks to see
     * if it has loaded that class before.  If it has it returns
     * a reference to it, if not, it loads it for the first time,
     * remembers it, then returns that.
     *
     * @param className the PluginServiceCheck to be loaded
     * @return the requested PluginServiceCheck
     */
    public synchronized PluginServiceCheck getServiceCheck(String className) {
         _logger.write(toString(), Logger.DEBUG, "got request for service check - " + className);
        // see if we've already got the service check loaded
        if(_loadedServiceChecks.containsKey(className)) {
            _logger.write(toString(), Logger.DEBUG, className + " already loaded - returning reference to it.");
            return (PluginServiceCheck) _loadedServiceChecks.get(className);
        }
        // otherwise we need to load it and return
        _logger.write(toString(), Logger.DEBUG, "attempting to load service check - " + className);
        
        // Create an instance of the specified PluginServiceCheck
        // then added it to the list of already loaded service checks
        PluginServiceCheck serviceCheck = null;
        try {
            serviceCheck = (PluginServiceCheck)ClassLoader.getSystemClassLoader().loadClass(className).newInstance();
            _loadedServiceChecks.put(className, serviceCheck);
            _logger.write(toString(), Logger.DEBUG, "loaded service check - "+className+" ("+serviceCheck.getDescription()+")");
        }
        catch (InstantiationException e){
            _logger.write(toString(), Logger.ERROR, "failed to instantiate - "+className);
            _logger.write(toString(), Logger.ERROR, e.getMessage());
        }
        catch (Exception e){
            _logger.write(toString(), Logger.ERROR, "failed to add - "+className);
            _logger.write(toString(), Logger.ERROR, e.toString());
        }
        
        return serviceCheck;
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---
 
    /**
     * A hash map containing host pipelines
     */
    private HashMap _hostPipelines = new HashMap();
 
    /**
     * A hash map containing host pipelines
     */
    private HashMap _loadedServiceChecks = new HashMap();
 
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = FilterMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();

//---STATIC ATTRIBUTES---

    /**
     * A reference to the single instance of this class
     */
    private static PluginServiceCheckManager _instance;
}
